@@ -1,4 +1,4 @@
-/* 11may13abu
+/* 01jan14abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -8,9 +8,9 @@ FormReq.upload.addEventListener("load", dropLoad, false);
 
 var Btn = [];
 var Queue = [];
-var SesId, Key, InBtn, Auto, Drop, Hint, Hints, Item, Beg, End;
+var SesId, Key, InBtn, Auto, Chg, Drop, Hint, Hints, Item, Beg, End;
 
-function inBtn(flg) {InBtn = flg;}
+function inBtn(btn,flg) {InBtn = flg;}
 
 function formKey(event) {
    Key = event.keyCode;
@@ -22,12 +22,15 @@ function formKey(event) {
          return false;
       }
    }
+   if (event.charCode || event.keyCode == 8)
+      Chg = true;
    return true;
 }
 
 function fldChg(field) {
+   Chg = true;
    if (!InBtn && Key != 13)
-      post(field.form, null, null);
+      post(field.form, false, null, null);
    return true;
 }
 
@@ -46,7 +49,7 @@ function doDrop(btn, event) {
    if (event.dataTransfer.files.length != 0) {
       Btn.push(Drop = btn);
       btn.value = "0 %";
-      post(btn.form, null, event.dataTransfer.files[0]);
+      post(btn.form, false, null, event.dataTransfer.files[0]);
    }
 }
 
@@ -83,18 +86,17 @@ function doPost(form) {
       if (i == Btn.length)
          return true;
       if (Btn[i].form == form)
-         break;
+         return post(form, false, null, null);
    }
-   return post(form, null, null);
 }
 
-function post(form, exe, file) {
+function post(form, auto, exe, file) {
    var i, data;
 
    if (!FormReq || !hasElement(form,"*Get") || (i = form.action.indexOf("~")) <= 0)
       return true;
    if (FormReq.readyState > 0 && FormReq.readyState < 4) {
-      Queue.push([form, exe, file]);
+      Queue.push([form, auto, exe, file]);
       return false;
    }
    form.style.cursor = "wait";
@@ -115,160 +117,180 @@ function post(form, exe, file) {
 
             if (Auto)
                clearTimeout(Auto);
-            if (r[1])
-               Auto = setTimeout("document.getElementById(\"" + r[0] + "\").click()", r[1]);
-         }
-         for (i = 1; i < txt.length;) {
-            var fld = txt[i++];
-            var val = decodeURIComponent(txt[i++]);
-
-            if (!fld) {
-               window[txt[i++]](val);
-               continue;
-            }
-            if (!(fld = document.getElementById(fld)))
-               continue;
-            if (fld.tagName == "SPAN") {
-               if (i != txt.length && txt[i].charAt(0) == "=")
-                  ++i;
-               if (i == txt.length || txt[i].charAt(0) != "+") {
-                  if (fld.firstChild.tagName != "A")
-                     fld.firstChild.data = val? val : "\u00A0";
-                  else
-                     fld.replaceChild(document.createTextNode(val? val : "\u00A0"), fld.firstChild);
-               }
-               else {
-                  var a = document.createElement("A");
-
-                  setHref(a, txt[i++].substr(1));
-                  a.appendChild(document.createTextNode(val));
-                  fld.replaceChild(a, fld.firstChild);
-               }
-            }
-            else if (fld.tagName == "A") {
-               if (i != txt.length && txt[i].charAt(0) == "=")
-                  ++i;
-               if (i == txt.length || txt[i].charAt(0) != "+") {
-                  fld.replaceChild(document.createTextNode(val? val : "\u00A0"), fld.firstChild);
-                  fld.removeAttribute("href");
-               }
-               else {
-                  fld.firstChild.data = val;
-                  setHref(fld, txt[i++].substr(1));
-               }
-            }
-            else if (fld.tagName == "IMG") {
-               var parent = fld.parentNode;
-
-               fld.src = val;
-               fld.alt = txt[i++];
-               if (parent.tagName == "A") {
-                  if (txt[i])
-                     setHref(parent, txt[i]);
-                  else {
-                     var grand = parent.parentNode;
-
-                     grand.removeChild(parent);
-                     grand.appendChild(fld);
-                  }
-               }
-               else if (txt[i]) {
-                  var a = document.createElement("A");
-
-                  parent.removeChild(fld);
-                  parent.appendChild(a);
-                  a.appendChild(fld);
-                  setHref(a, txt[i]);
-               }
-               ++i;
-            }
+            if (!r[1])
+               Auto = null;
             else {
-               if (fld.type == "checkbox") {
-                  fld.checked = val != "";
-                  document.getElementsByName(fld.name)[0].value = "";
-               }
-               else if (fld.type == "select-one") {
-                  for (j = 0; j < fld.options.length; ++j) {
-                     if (fld.options[j].text == val)
-                        fld.selectedIndex = j;
-                     fld.options[j].disabled = false;
+               Auto = setTimeout(function() {
+                  if (Chg)
+                     Auto = setTimeout(arguments.callee, r[1]);
+                  else {
+                     Btn.push(document.getElementById(r[0]));
+                     post(form, true, null, null);
                   }
-               }
-               else if (fld.type == "radio") {
-                  fld.value = val;
-                  fld.checked = txt[i++].charAt(0) != "";
-               }
-               else if (fld.type == "image")
-                  fld.src = val;
-               else if (fld.value != val) {
-                  fld.value = val;
-                  fld.scrollTop = fld.scrollHeight;
-               }
-               fld.disabled = false;
-               if (i < txt.length && txt[i].charAt(0) == "=") {
-                  if (fld.type == "select-one") {
-                     for (j = 0; j < fld.options.length; ++j)
-                        if (fld.options[j].text != val)
-                           fld.options[j].disabled = true;
-                  }
-                  fld.disabled = true;
-                  InBtn = 0;  // 'onblur' on won't come when disabled
-                  if (fld.type == "checkbox"  &&  fld.checked)
-                     document.getElementsByName(fld.name)[0].value = "T";
-                  ++i;
-               }
+               }, r[1] );
             }
-            while (i < txt.length && (j = "#*?".indexOf(txt[i].charAt(0))) >= 0) {
-               switch (j) {
+         }
+         if (!auto || !Chg) {
+            for (i = 1; i < txt.length;) {
+               var fld = txt[i++];
+               var val = decodeURIComponent(txt[i++]);
 
-               case 0:  // '#'
-                  var cls;
-
-                  val = txt[i++].substr(1);
-                  if ((cls = fld.getAttribute("class")) != null  &&  (j = cls.indexOf(" ")) >= 0)
-                     val += cls.substr(j);
-                  fld.setAttribute("class", val);
-                  break;
-
-               case 1:  // '*'
-                  var node = fld.parentNode.parentNode.lastChild;
-                  var img = document.createElement("IMG");
-
-                  if (!node.firstChild)
-                     node = fld.parentNode.parentNode.parentNode.lastChild;
-                  node.removeChild(node.firstChild);
-                  img.src = txt[i++].substr(1);
-                  if (!txt[i])
-                     node.appendChild(img);
+               if (!fld) {
+                  window[txt[i++]](val);
+                  continue;
+               }
+               if (!(fld = document.getElementById(fld)))
+                  continue;
+               if (fld.tagName == "SPAN") {
+                  if (i != txt.length && txt[i].charAt(0) == "=")
+                     ++i;
+                  if (i == txt.length || txt[i].charAt(0) != "+") {
+                     if (fld.firstChild.tagName != "A")
+                        fld.firstChild.data = val? val : "\u00A0";
+                     else
+                        fld.replaceChild(document.createTextNode(val? val : "\u00A0"), fld.firstChild);
+                  }
                   else {
                      var a = document.createElement("A");
 
+                     setHref(a, txt[i++].substr(1));
+                     a.appendChild(document.createTextNode(val));
+                     fld.replaceChild(a, fld.firstChild);
+                  }
+               }
+               else if (fld.tagName == "A") {
+                  if (i != txt.length && txt[i].charAt(0) == "=")
+                     ++i;
+                  if (i == txt.length || txt[i].charAt(0) != "+") {
+                     fld.replaceChild(document.createTextNode(val? val : "\u00A0"), fld.firstChild);
+                     fld.removeAttribute("href");
+                  }
+                  else {
+                     fld.firstChild.data = val;
+                     setHref(fld, txt[i++].substr(1));
+                  }
+               }
+               else if (fld.tagName == "IMG") {
+                  var parent = fld.parentNode;
+
+                  fld.src = val;
+                  fld.alt = txt[i++];
+                  if (parent.tagName == "A") {
+                     if (txt[i])
+                        setHref(parent, txt[i]);
+                     else {
+                        var grand = parent.parentNode;
+
+                        grand.removeChild(parent);
+                        grand.appendChild(fld);
+                     }
+                  }
+                  else if (txt[i]) {
+                     var a = document.createElement("A");
+
+                     parent.removeChild(fld);
+                     parent.appendChild(a);
+                     a.appendChild(fld);
                      setHref(a, txt[i]);
-                     a.appendChild(img);
-                     node.appendChild(a);
                   }
                   ++i;
-                  break;
+               }
+               else {
+                  if (fld.type == "checkbox") {
+                     fld.checked = val != "";
+                     document.getElementsByName(fld.name)[0].value = "";
+                  }
+                  else if (fld.type == "select-one") {
+                     for (j = 0; j < fld.options.length; ++j) {
+                        if (fld.options[j].text == val)
+                           fld.selectedIndex = j;
+                        fld.options[j].disabled = false;
+                     }
+                  }
+                  else if (fld.type == "radio") {
+                     fld.value = val;
+                     fld.checked = txt[i++].charAt(0) != "";
+                  }
+                  else if (fld.type == "image")
+                     fld.src = val;
+                  else if (fld.value != val) {
+                     fld.value = val;
+                     fld.scrollTop = fld.scrollHeight;
+                  }
+                  fld.disabled = false;
+                  if (i < txt.length && txt[i].charAt(0) == "=") {
+                     if (fld.type == "select-one") {
+                        for (j = 0; j < fld.options.length; ++j)
+                           if (fld.options[j].text != val)
+                              fld.options[j].disabled = true;
+                     }
+                     fld.disabled = true;
+                     InBtn = 0;  // 'onblur' on won't come when disabled
+                     if (fld.type == "checkbox"  &&  fld.checked)
+                        document.getElementsByName(fld.name)[0].value = "T";
+                     ++i;
+                  }
+               }
+               while (i < txt.length && (j = "#*?".indexOf(txt[i].charAt(0))) >= 0) {
+                  switch (j) {
 
-               case 2:  // '?'
-                  fld.title = decodeURIComponent(txt[i++].substr(1));
-                  break;
+                  case 0:  // '#'
+                     var cls;
+
+                     val = txt[i++].substr(1);
+                     if ((cls = fld.getAttribute("class")) != null) {
+                        j = cls.indexOf("  ");
+                        if (!val)
+                           val = j >= 0? cls.substr(j+2) : cls;
+                        else if (j >= 0)
+                           val += cls.substr(j);
+                        else
+                           val += "  " + cls;
+                     }
+                     fld.setAttribute("class", val);
+                     break;
+
+                  case 1:  // '*'
+                     var node = fld.parentNode.parentNode.lastChild;
+                     var img = document.createElement("IMG");
+
+                     if (!node.firstChild)
+                        node = fld.parentNode.parentNode.parentNode.lastChild;
+                     node.removeChild(node.firstChild);
+                     img.src = txt[i++].substr(1);
+                     if (!txt[i])
+                        node.appendChild(img);
+                     else {
+                        var a = document.createElement("A");
+
+                        setHref(a, txt[i]);
+                        a.appendChild(img);
+                        node.appendChild(a);
+                     }
+                     ++i;
+                     break;
+
+                  case 2:  // '?'
+                     fld.title = decodeURIComponent(txt[i++].substr(1));
+                     break;
+                  }
                }
             }
+            Chg = false;
          }
       }
       form.style.cursor = "";
       if (Queue.length > 0) {
          var a = Queue.shift();
-         post(a[0], a[1], a[2]);
+         post(a[0], a[1], a[2], a[3]);
       }
    }
    if (!exe)
       data = "";
    else {
-      data = "*Gui:0:=" + exe[0];
+      data = "*Gui:0=" + exe[0];
       for (var i = 1; i < exe.length; ++i)
-         data += "&*JsArgs:" + i + ":=" + exe[i];
+         data += "&*JsArgs:" + i + "=" + exe[i];
    }
    for (i = 0; i < Btn.length;)
       if (Btn[i].form != form)
@@ -301,7 +323,7 @@ function post(form, exe, file) {
          var rd = new FileReader();
          rd.readAsBinaryString(file);
          rd.onload = function() {
-            FormReq.setRequestHeader("X-Pil", "*ContLen=");
+            FormReq.setRequestHeader("X-Pil", "*ContL=T");
             FormReq.sendAsBinary(data + "&*Drop=" +
                encodeURIComponent(file.name) + "=" +
                file.size + "\n" + rd.result );
@@ -335,9 +357,9 @@ function doHint(field) {
    field.onblur = function() {
       Hint.style.visibility = "hidden";
    }
-   var top = field.offsetHeight + 2;
-   var left = 3;
-   for (var obj = field;  obj.id != "main" && obj.id != "menu";  obj = obj.offsetParent) {
+   var top = field.offsetHeight;
+   var left = 0;
+   for (var obj = field;  obj && obj.style.position != "absolute";  obj = obj.offsetParent) {
       top += obj.offsetTop;
       left += obj.offsetLeft;
    }
@@ -463,7 +485,8 @@ function addHint(i, field, str) {
 function setHint(field, item) {
    Hint.style.visibility = "hidden";
    field.value = field.value.substr(0,Beg) + item.firstChild.nodeValue + field.value.substr(End);
-   post(field.form, null, null);
+   Chg = true;
+   post(field.form, false, null, null);
    field.setSelectionRange(Beg + item.firstChild.nodeValue.length, field.value.length);
    field.focus();
    field.onchange = function() {
@@ -483,6 +506,30 @@ function hintOff(i) {
    s.color= "black";
 }
 
+/*** Scroll/touch ***/
+var TblY;
+
+function tblTouch(event) {
+   if (event.touches.length == 1)
+      TblY = event.touches[0].pageY;
+   return true;
+}
+
+function tblMove(table, event) {
+   if (event.touches.length == 1) {
+      var dy = event.touches[0].pageY - TblY;
+
+      if (dy < -12 || dy > +12) {
+         TblY = event.touches[0].pageY;
+         for (var obj = table.parentNode;  obj;  obj = obj.parentNode)
+            if (obj.tagName == "FORM")
+               return post(obj, false, [dy > 6? "jsUp" : "jsDn"], null);
+      }
+      return false;
+   }
+   return true;
+}
+
 /*** Lisp calls ***/
 function lisp(form, fun) {
    if (form) {
@@ -493,7 +540,7 @@ function lisp(form, fun) {
             exe[i-1] = "+" + arguments[i];
          else
             exe[i-1] = "." + encodeURIComponent(arguments[i]);
-      return post(form, exe, null);
+      return post(form, false, exe, null);
    }
    if (arguments.length > 2) {
       fun += "?" + lispVal(arguments[2]);
@@ -503,12 +550,15 @@ function lisp(form, fun) {
    var req = new XMLHttpRequest();
    try {req.open("GET", SesId + "!" + fun);}
    catch (e) {return true;}
+   req.onload = function() {
+      if (req.responseText)
+         eval(req.responseText);
+   }
    try {req.send(null);}
    catch (e) {
       req.abort();
       return true;
    }
-   setTimeout(function(){req.abort();}, 40);  // No response expected
    return false;
 }
 
