@@ -1,4 +1,4 @@
-/* 20mar12abu
+/* 04feb13abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -15,6 +15,8 @@
 #include <sys/stat.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#include <netinet/in.h>
+#include <sys/socket.h>
 
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
